﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

using CEIP = ComiLib.cEIP.SafeNativeMethods;

/*
이 예제는 ceAI08A와 같은 Analog Input 모듈을 사용해 아날로그 입력을 받는 방법에 대한 예제입니다.
cEIP 모듈을 Load한 후 0채널~3채널까지 총 4개의 채널에 대해 아날로그 입력을 지속적으로 받습니다.
설정 변경을 통해 Range Mode, 전압-전류 입력모드를 설정할 수 있습니다.
*/


namespace cEIP_Analog_Input
{
    public partial class cEIP_Analog_Input : Form
    {
        private static int selected_channel;
        private static int max_channel;

        private static int range_mode;
        private static int voltage_current;

        private static double value_channel0;
        private static double value_channel1;
        private static double value_channel2;
        private static double value_channel3;

        private static int ErrorCode;

        private static int isSearchedDev;
        private static int TotalNodes;


        private enum voltage_current_mode
        {
            Voltage,
            Current
        }

        public cEIP_Analog_Input()
        {
            InitializeComponent();
        }

        private void cEIP_AI_Example_Load(object sender, EventArgs e)
        {
            Load_cEIP();
            example_init();
        }

        private void Load_cEIP() //cEIP 모듈을 불러오기
        {
            TotalNodes = 0;
            isSearchedDev = 0;


            ErrorCode = CEIP.ceGnLoad(); //cEIP Load
            if(ErrorCode == CEIP.ceERR_NONE) 
            {
                ErrorCode = CEIP.ceGnIsSearchedDevice(ref isSearchedDev);
                show_error_code();

                if (isSearchedDev == 0)
                {
                    CEIP.ceGnSearchDevice(5, 200, CEIP.CE_TRUE, ref TotalNodes);
                    show_error_code();
                }
                else
                {
                    CEIP.ceGnReSearchDevice(5, 200, CEIP.CE_TRUE, ref TotalNodes);
                    show_error_code();
                }

                if (TotalNodes == 0)
                {
                    MessageBox.Show("Node가 없습니다.");
                }

                ErrorCode = CEIP.ceGnTotalAIChannel(ref max_channel);
                show_error_code();
            }
            else
            {
                show_error_code();
            }
        }
        
        private void read_AIValue() // 전압 / 전류값 읽어오기
        {
            if(voltage_current == (int)voltage_current_mode.Voltage)
            {
                ErrorCode = CEIP.ceaiVolt_Get(0, ref value_channel0);
                show_error_code();
                ErrorCode = CEIP.ceaiVolt_Get(1, ref value_channel1);
                show_error_code();
                ErrorCode = CEIP.ceaiVolt_Get(2, ref value_channel2);
                show_error_code();
                ErrorCode = CEIP.ceaiVolt_Get(3, ref value_channel3);
                show_error_code();
            }
            else if(voltage_current == (int)voltage_current_mode.Current) 
            {
                ErrorCode = CEIP.ceaiCurrent_Get(0, ref value_channel0);
                show_error_code();
                ErrorCode = CEIP.ceaiCurrent_Get(1, ref value_channel1);
                show_error_code();
                ErrorCode = CEIP.ceaiCurrent_Get(2, ref value_channel2);
                show_error_code();
                ErrorCode = CEIP.ceaiCurrent_Get(3, ref value_channel3);
                show_error_code();
            }
        }
        private void Analog_Input_Checking_Timer_Tick(object sender, EventArgs e)
        {
            read_AIValue();
            refresh_Inputs();
        }

        private void cEIP_AI_Example_FormClosing(object sender, FormClosingEventArgs e)
        {
            Analog_Input_Checking_Timer.Stop();
            CEIP.ceGnUnload();
        }

        private void CB_channel_SelectedIndexChanged(object sender, EventArgs e)
        {
            selected_channel = CB_channel.SelectedIndex;
            refresh_Comboboxes();
        }

        private void CB_rangemode_SelectedIndexChanged(object sender, EventArgs e)
        {
            range_mode = CB_rangemode.SelectedIndex;
            ErrorCode = CEIP.ceaiVoltRangeMode_Set(selected_channel, range_mode);
            show_error_code();
            refresh_Comboboxes();
        }

        private void CB_voltcur_SelectedIndexChanged(object sender, EventArgs e) 
        {
            voltage_current = CB_voltcur.SelectedIndex;
            refresh_Comboboxes();
        }

        private void example_init() // 예제 시작전 초기화
        {
            selected_channel = 0;
            range_mode = 0;
            voltage_current = (int)voltage_current_mode.Voltage;

            value_channel0 = 0;
            value_channel1 = 0;
            value_channel2 = 0;
            value_channel3 = 0;

            for (int i = 0; i < max_channel; i++)
            {
                CB_channel.Items.Add(i);
            }

            CB_channel.SelectedIndex = selected_channel;
            CB_rangemode.SelectedIndex = range_mode;
            CB_voltcur.SelectedIndex = voltage_current;

            refresh_Inputs();

            Analog_Input_Checking_Timer.Start();
        }
        private void refresh_Comboboxes() // UI 업데이트 - 콤보박스부분
        {
            CEIP.ceaiVoltRangeMode_Get(selected_channel, ref range_mode);

            CB_channel.SelectedIndex = selected_channel;
            CB_rangemode.SelectedIndex = range_mode;
            CB_voltcur.SelectedIndex = voltage_current;
        }
        private void refresh_Inputs() // UI 업데이트 - AI부분
        {
            string suffix = voltage_current == (int)voltage_current_mode.Voltage ? "V" : "mA";

            LB_channel0.Text = "Channel 0 - " + value_channel0.ToString("N5") + suffix;
            LB_channel1.Text = "Channel 1 - " + value_channel1.ToString("N5") + suffix;
            LB_channel2.Text = "Channel 2 - " + value_channel2.ToString("N5") + suffix;
            LB_channel3.Text = "Channel 3 - " + value_channel3.ToString("N5") + suffix;
        }
        private void show_error_code() //에러코드 표시
        {
            if (ErrorCode != CEIP.ceERR_NONE)
            {
                MessageBox.Show("Error Code - " + ErrorCode);
            }
            ErrorCode = CEIP.ceERR_NONE;
        }
    }
}
