﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Threading;

using CEIP = ComiLib.cEIP.SafeNativeMethods;

namespace cEIP_Digital_Input
{
    public partial class cEIP_Digital_Input : Form
    {
        const int nRealNode = 10; // 이 값은 실제 노드(ceNM-SE) 개수를 입력하면 됩니다.
        const int nTimeOut = (10 * 100); // 노드 하나당 100ms
        int nInitCh = 0;
        const int nNumCh = 32;
        //bool bIsDeviceAvailable = false;
        int nIsSearchedDev = 0;
        int nNumNodes = 0;
        
        //int nNumDioNodes = 0;
        int nNodeID = 100;
        uint nDiState = 0;
        int nRtn = 0;
        Button[] DiState;
        bool bIsLogicMode = false; // false 면 DI State 확인, True면 Logic State 확인
        int nLogicMask = 0;

        public cEIP_Digital_Input()
        {
            InitializeComponent();

            DiState = new Button[nNumCh]
            {
                btnDI0,btnDI1,btnDI2,btnDI3,btnDI4,btnDI5,btnDI6,btnDI7,btnDI8,btnDI9,btnDI10,
                btnDI11,btnDI12,btnDI13,btnDI14,btnDI15,btnDI16,btnDI17,btnDI18,btnDI19,btnDI20,
                btnDI21,btnDI22,btnDI23,btnDI24,btnDI25,btnDI26,btnDI27,btnDI28,btnDI29,btnDI30,
                btnDI31,
            };
        }

        private void cEIP_DI_Load(object sender, EventArgs e)
        {
            nRtn = CEIP.ceGnLoad(); //
            if (nRtn != CEIP.ceERR_NONE) // 라이브러리가 로드 된 상태에서 라이브러리 사용을 위한 구동 초기화 역할을 합니다
                GetLog("Library Initialize Failed, follow the ErrorCode");
            else
            {
                lbLogView.Items.Add("Library Initialize Success");
            }
        }

        private void btnNodeScan_Click(object sender, EventArgs e)
        {
            nInitCh = Convert.ToInt32(textBox_InitCh.Text);
            NodeScan();
            FindDioNodes();
            timer_DICheck.Enabled = true;
        }

        internal void NodeScan()
        {
            //timer_DICheck.Enabled = false;
            nRtn = CEIP.ceGnIsSearchedDevice(ref nIsSearchedDev); // 노드 탐색 완료 여부 확인, 노드 탐색 전이라면 ceGnSearchDevice, 노드 탐색 후라면 ceGnReSearchEdvice 사용
            if (nIsSearchedDev == 0) // 노드 검색이 기존에 완료되지 않음
            {
                if ((nRtn = CEIP.ceGnSearchDevice(nRealNode, nTimeOut, 0, ref nNumNodes)) < 0)
                    GetLog("Can Not Find Any Nodes");
                else
                    GetLog(nNumNodes.ToString() + " Node(s) Find");                    
            }
            else
            {
                if ((nRtn = CEIP.ceGnReSearchDevice(nRealNode, nTimeOut, 0, ref nNumNodes)) < 0)
                    GetLog("Can Not Find Any Nodes");
                else
                    GetLog(nNumNodes.ToString() + " Node(s) Find");
            }
        }

        internal void FindDioNodes()
        {
            int nDIOModuleCnt = 0;
            int nNumDIOCh = 0;


            nRtn = CEIP.ceGnTotalDIOChannel(ref nNumDIOCh);
            if (nRtn == 0)
                GetLog("DIO Channel Count : " + nNumDIOCh.ToString());
            else
                GetLog("ceGnTotalDIOChannel Failed -" + nRtn);
        }

        private void timer_DICheck_Tick(object sender, EventArgs e)
        {
            if (bIsLogicMode == false)
            {
                nRtn = CEIP.cedioMulti_Get(nInitCh, nNumCh, ref nDiState);
                for (int i = 0; i < nNumCh; i++)
                {
                    DiState[i].Enabled = false;

                    if (((nDiState >> i) & 0x1) == 1) // ON
                    {
                        DiState[i].BackColor = Color.DarkCyan;
                        DiState[i].Text = (i + nInitCh).ToString();
                    }
                    else
                    {
                        DiState[i].BackColor = Color.Transparent;
                        DiState[i].Text = (i + nInitCh).ToString();
                    }                    
                }
            }
            else
            {                
                nRtn = CEIP.cedioLogicMulti_Get(nInitCh, nNumCh, ref nLogicMask);                
                for (int i = 0; i < nNumCh; i++)
                {
                    DiState[i].Enabled = true;

                    if (((nLogicMask >> i) & 0x1) == (int)CEIP.SigLogic.LOGIC_A) //LOGIC A
                    {
                        DiState[i].BackColor = Color.Transparent;
                        DiState[i].Text = (i + nInitCh).ToString() + " A";
                    }
                    else
                    {
                        DiState[i].BackColor = Color.DarkCyan;
                        DiState[i].Text = (i + nInitCh).ToString() + " B";
                    }                    
                }
            }
        }

        private void btnModeChange_Click(object sender, EventArgs e)
        {
            bIsLogicMode = !bIsLogicMode;
            btnModeChange.Text = bIsLogicMode == true ? "Logic State" : "DI State";
        }

        internal void btnDI_Clicked(object sender, EventArgs e)
        {
            int nSelCh = Convert.ToInt16(((Button)sender).Tag) + nInitCh;
            int nSelChLogic = (((nLogicMask >> (nSelCh - nInitCh)) & 0x1) == (int)CEIP.SigLogic.LOGIC_A ? (int)CEIP.SigLogic.LOGIC_B : (int)CEIP.SigLogic.LOGIC_A);
            nRtn = CEIP.cedioLogicOne_Set(nSelCh, nSelChLogic);
            if (nRtn != CEIP.ceERR_NONE)
                GetLog("Ch " + nSelCh.ToString() + " Logic Change Failed : " + nRtn.ToString());
            else
                GetLog("Ch " + nSelCh.ToString() + " Logic Changed");
        }

        private void cEIP_DI_FormClosing(object sender, FormClosingEventArgs e)
        {
            timer_DICheck.Enabled = false;
            nRtn = CEIP.ceGnUnload();
            Dispose();
        }

        internal void GetLog(string sLog)
        {
            lbLogView.Items.Add(sLog);
            lbLogView.SelectedIndex = lbLogView.Items.Count - 1;
        }
    }
}
