﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using CEIP = ComiLib.cEIP.SafeNativeMethods;

// 본 프로그램은 MADIC.exe 를 띄워 놓으신 후 모니터링 하시면서 동시에 구동 해 보시면 동작 확인에 유용합니다.
namespace cEIP_SingleAxisMotion
{
    public partial class cEIP_SingleAxisMotion : Form
    {
        const int EXPECTED_NODECOUNT = 10; // 이 값은 실제 노드(ceNM-SE) 개수를 입력하면 됩니다.
        const int TIMEOUT = (10 * 100); // 노드 하나당 100ms

        bool g_bIsDeviceAvailable = false;

        public cEIP_SingleAxisMotion()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            int nResult = CEIP.ceERR_NONE; // ceERR_NONE(0) 은 실패가 아니라 Error 가 없음을 의미, 에러 발생 시 마이너스(-) 값으로 에러 코드 반환

            // 디바이스 로드
            nResult = CEIP.ceGnLoad();
            if (nResult != CEIP.ceERR_NONE)
            {
                MessageBox.Show("ceGnLoad 실패, 에러코드는 nResult 참고");
            }
            else // 디바이스 로드 성공
            {
                int nTotalNodes = 0;
                CEIP.TNode TNode= new CEIP.TNode();
                CEIP.cemAdvGetNodeInformation(0, ref TNode);
                CEIP.ceGnTotalNode(ref nTotalNodes);
                int nTempAxes = TNode.TotalMotModule;

                CEIP.TNodeInformation TNodeInformation = new CEIP.TNodeInformation();
                CEIP.cemAdvGetAllNodeInformation(ref TNodeInformation);
                RescanAndUpdate();
                CEIP.ceGnTotalNode(ref nTotalNodes);
            }
        }

        private void btnRescan_Click(object sender, EventArgs e)
        {
            RescanAndUpdate();
        }

        private bool RescanAndUpdate()
        {
            // 타이머 동작을 먼저 멈춘다.(노드 재스캔 중 모션 함수 수행 금지)
            timer1.Enabled = false;

            int nResult = CEIP.ceERR_NONE; // ceERR_NONE(0) 은 실패가 아니라 Error 가 없음을 의미, 에러 발생 시 마이너스(-) 값으로 에러 코드 반환

            int nIsSearchedDev = 0;
            int nNumNodes = 0;

            //이미 검색된 노드인지 확인합니다.
            CEIP.ceGnIsSearchedDevice(ref nIsSearchedDev);

            if (nIsSearchedDev == 0) // 노드 검색이 기존에 완료 안됨
            {
                //노드를 검색합니다.
                if ((nResult = CEIP.ceGnSearchDevice(EXPECTED_NODECOUNT, TIMEOUT, 0, ref nNumNodes)) < 0)
                {
                    //ShowCeSdkError(nResult);
                    timer1.Enabled = false;
                    return false;
                }
            }
            else
            {
                //노드를 재검색합니다.
                if ((nResult = CEIP.ceGnReSearchDevice(EXPECTED_NODECOUNT, TIMEOUT, 0, ref nNumNodes)) < 0)
                {
                    //ShowCeSdkError(nResult);
                    timer1.Enabled = false;
                    return false;
                }
            }

            int nNodes = 0;
            int nNumAxes = 0;

            CEIP.ceGnTotalNode(ref nNodes);

            CEIP.ceGnTotalMotionChannel(ref nNumAxes);

            // UI 업데이트
            string sAxis;
            cmbAxes.Items.Clear();
            for (int i = 0; i < nNumAxes; i++)
            {
                sAxis = "Axis" + i;
                cmbAxes.Items.Add(sAxis);
            }

            if (nNumAxes > 0)
            {
                g_bIsDeviceAvailable = true;
                MessageBox.Show("노드 스캔 및 모션 모듈 인식 성공");
                cmbAxes.SelectedIndex = 0;
            }
            else
            {
                g_bIsDeviceAvailable = false;
                MessageBox.Show("모션 모듈이 하나도 인식되지 않았습니다. 노드 연결 및 IP 설정을 점검하거나 스캔을 다시 시도하세요.");
                timer1.Enabled = false;

                return false;
            }

            // 각종 초기화 및 로직 설정 => 기본적으로 서보알람과, EL(+) 및 EL(-) 센서가 OFF 상태이어야 CW 또는 CCW 방향으로 이송이 가능합니다. EL 센서인 경우는 해당 이송방향에 대한 +/-EL 상태에 따라 이송 안됨.
            // 0번축 설정
            CEIP.cemCfgMioProperty_Set(0, (int)CEIP.MioPropId.ALM_LOGIC, (int)CEIP.SigLogic.LOGIC_A); // 알람 로직은 보통 Logic_B 입니다.
            CEIP.cemCfgMioProperty_Set(0, (int)CEIP.MioPropId.EL_LOGIC, (int)CEIP.SigLogic.LOGIC_A); // 리미트센서 로직은 보통 Logic_B 입니다.
            CEIP.cemCfgMioProperty_Set(0, (int)CEIP.MioPropId.INP_EN, 0); // 서보로부터 INP(Inposition) 신호를 받지 않도록 설정. 만일 이 값이 Enable(1) 이면 Command Pulse 가 
                                                                                      // 모두 출력이 되더라도 서보로부터 INP 를 받아야 이송완료로 처리되며 그렇지 않을 경우 이송완료를 무한 기다리는 상태가 되므로 주의.
            CEIP.cemCfgOutMode_Set(0, (int)CEIP._TCemOutMode.cemOMODE_CWCCW0); // 모터 드라이버의 지령펄스 입력 모드와 동일하게 맞춰야 펄스 및 방향이 올바르게 동작합니다.
            CEIP.cemCfgInMode_Set(0, (int)CEIP._TCemInMode.cemIMODE_AB4X, 0); // 모터 드라이버로부터의 피드백 펄스에 채배(1, 2 또는 4채배 가능)를 결정합니다.

            CEIP.cemCfgFilter_Set(0, 1); // ORG, EL 등 센서 신호에 대해 노이즈 필터링 할 수 있도록 처리
            CEIP.cemCfgFilterAB_Set(0, (int)CEIP._TCemABFilter.cemAB_ENC, 1); // 엔코더 입력 EA, EB 신호에 대해 노이즈 필터링 할 수 있도록 처리

            // 1번축 설정
            CEIP.cemCfgMioProperty_Set(1, (int)CEIP.MioPropId.ALM_LOGIC, (int)CEIP.SigLogic.LOGIC_A); // 알람 로직은 보통 Logic_B 입니다.
            CEIP.cemCfgMioProperty_Set(1, (int)CEIP.MioPropId.EL_LOGIC, (int)CEIP.SigLogic.LOGIC_A); // 리미트센서 로직은 보통 Logic_B 입니다.
            CEIP.cemCfgMioProperty_Set(1, (int)CEIP.MioPropId.INP_EN, 0); // 서보로부터 INP(Inposition) 신호를 받지 않도록 설정. 만일 이 값이 Enable(1) 이면 Command Pulse 가 
                                                                                    // 모두 출력이 되더라도 서보로부터 INP 를 받아야 이송완료로 처리되며 그렇지 않을 경우 이송완료를 무한 기다리는 상태가 되므로 주의.
            CEIP.cemCfgOutMode_Set(1, (int)CEIP._TCemOutMode.cemOMODE_CWCCW0); // 모터 드라이버의 지령펄스 입력 모드와 동일하게 맞춰야 펄스 및 방향이 올바르게 동작합니다.
            CEIP.cemCfgInMode_Set(1, (int)CEIP._TCemInMode.cemIMODE_AB4X, 0); // 모터 드라이버로부터의 피드백 펄스에 채배(1, 2 또는 4채배 가능)를 결정합니다.

            CEIP.cemCfgFilter_Set(1, 1); // ORG, EL 등 센서 신호에 대해 노이즈 필터링 할 수 있도록 처리
            CEIP.cemCfgFilterAB_Set(1, (int)CEIP._TCemABFilter.cemAB_ENC, 1); // 엔코더 입력 EA, EB 신호에 대해 노이즈 필터링 할 수 있도록 처리

            // 서보온
            CEIP.cemGnServoOn_Set(0, 1); // 0번축 서보온
            CEIP.cemGnServoOn_Set(1, 1); // 1번축 서보온

            timer1.Enabled = true;

            return true;
        }

        private void btnResetPos_Click(object sender, EventArgs e)
        {
            int nAxis = cmbAxes.SelectedIndex;

            CEIP.cemStPosition_Set(nAxis, (int)CEIP._TCemCntr.cemCNT_COMM, 0); // 두번째 파라미터: 0(Command Pulse), 1(Feedback Pulse)
            CEIP.cemStPosition_Set(nAxis, (int)CEIP._TCemCntr.cemCNT_FEED, 0); // 두번째 파라미터: 0(Command Pulse), 1(Feedback Pulse)

            //CEIP.cemStCount_Get(nAxis, (int)CEIP._TCemCntr.cemCNT_COMM, 0); // 두번째 파라미터: 0(Command Pulse), 1(Feedback Pulse)
            //CEIP.cemStCount_Get(nAxis, (int)CEIP._TCemCntr.cemCNT_FEED, 0); // 두번째 파라미터: 0(Command Pulse), 1(Feedback Pulse)
        }

        private void btnMoveN_Click(object sender, EventArgs e)
        {
            int nAxis = cmbAxes.SelectedIndex;
            CEIP.cemCfgSpeedPattern_Set(nAxis, (int)CEIP._TCemSpeedMode.cemSMODE_S/*S_Curve*/, 10000, 100000, 100000);
            CEIP.cemSxMoveStart(nAxis, -50000); // cemSxMove 함수 사용시 이송 완료시까지 함수 반환이 되지 않습니다. cemSxMoveStart 함수처럼 끝에 Start 가 붙은 함수는 이송 완료시까지 기다리지 않고 이송 명령만 내린 후 바로 반환 됩니다.
            // 보통은 MoveStart 함수는 WaitDone 함수와 쌍으로 사용 됩니다.
        }

        private void btnMoveP_Click(object sender, EventArgs e)
        {
            int nAxis = cmbAxes.SelectedIndex;

            CEIP.cemCfgSpeedPattern_Set(nAxis, (int)CEIP._TCemSpeedMode.cemSMODE_T/*Trapezoidal*/, 10000, 100000, 100000);

#if true
            CEIP.cemSxMoveStart(nAxis, 50000); // cemSxMove 함수 사용시 이송 완료시까지 함수 반환이 되지 않습니다. cemSxMoveStart 함수처럼 끝에 Start 가 붙은 함수는 이송 완료시까지 기다리지 않고 이송 명령만 내린 후 바로 반환 됩니다.
            // 보통은 MoveStart 함수는 WaitDone 함수와 쌍으로 사용 됩니다.
#endif

#if false // 참고 코드
            CEIP.cemSxMove(nAxis, 5000, false); // 이송 완료시까지 함수 반환이 안되고 기다림.        
#endif

#if false // 참고 코드
            CEIP.cemSxMoveStart(nAxis, 5000);
            int nMotStates = 0;
            int nMioStates = 0;
            while (true)
            {
                CEIP.cemStReadMotionState(nAxis, ref nMotStates);
                CEIP.cemStReadMioStatuses(nAxis, ref nMioStates);

                if (nMotStates == (int)CEIP._TCemMotionState.cemMST_STOP) // stop
                {
                    break;
                }

                if (((nMioStates >> (int)CEIP._TCemMioState.cemIOST_ALM) & 0x1) == 1 || ((nMioStates >> (int)CEIP._TCemMioState.cemIOST_ELP) & 0x1) == 1
                    || ((nMioStates >> (int)CEIP._TCemMioState.cemIOST_ELN) & 0x1) == 1) // ALM 또는 EL(+/-) 이 ON 인지 확인
                {
                    // 알람 또는 리미트 센서 감지 됨.
                    break;
                }
            }
#endif

        }

        private void btnQuit_Click(object sender, EventArgs e)
        {
            // 디바이스 언로드
            CEIP.ceGnUnload();
            timer1.Enabled = false;
            Dispose();
        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            if (!g_bIsDeviceAvailable) return;

            int nAxis = cmbAxes.SelectedIndex;

            // command(지령) 펄스 및 피드백 펄스 위치 업데이트
            double fCmdPos=0, fFeedPos=0;
            CEIP.cemStPosition_Get(nAxis, (int)CEIP._TCemCntr.cemCNT_COMM, ref fCmdPos);
            CEIP.cemStPosition_Get(nAxis, (int)CEIP._TCemCntr.cemCNT_FEED, ref fFeedPos);
            
            lblCmdPos.Text = fCmdPos.ToString();
            lblFeedPos.Text = fFeedPos.ToString();

            // 모션 상태 업데이트
            int nMotStates = 0;
            int nMioStates = 0;
            char []abyMst = new char[500];
            string strMst = "";

            CEIP.cemStReadMotionState(nAxis, ref nMotStates);
            CEIP.cemStReadMioStatuses(nAxis, ref nMioStates);

            CEIP.cmmStGetMstString(nMotStates, abyMst, 500);
            strMst = "";
            for (int j = 0; j < abyMst.Length; j++)
            {
                strMst += Convert.ToChar(abyMst[j]);
            }

            lblMotStatus.Text = strMst;

            // MIO(Machine I/O) 상태 업데이트
            chkRDY.Checked = Convert.ToBoolean((nMioStates >> (int)CEIP.MioState.RDY) & 0x1);
            chkAlm.Checked = Convert.ToBoolean((nMioStates >> (int)CEIP.MioState.ALM) & 0x1);
            chkELP.Checked = Convert.ToBoolean((nMioStates >> (int)CEIP.MioState.ELP) & 0x1);
            chkELN.Checked = Convert.ToBoolean((nMioStates >> (int)CEIP.MioState.ELN) & 0x1);
            chkORG.Checked = Convert.ToBoolean((nMioStates >> (int)CEIP.MioState.ORG) & 0x1);
            chkEZ.Checked  = Convert.ToBoolean((nMioStates >> (int)CEIP.MioState.EZ)  & 0x1);
            chkINP.Checked = Convert.ToBoolean((nMioStates >> (int)CEIP.MioState.INP) & 0x1);

            // 원점복귀인 경우 busy(아직 원점 복귀 중)상태 표시
            int isBusy = 0;
            int isSuccess = 0;

            CEIP.cemHomeIsBusy(nAxis, ref isBusy);
            chkHomeIsBusy.Checked = Convert.ToBoolean(isBusy);

            CEIP.cemHomeSuccess_Get(nAxis, ref isSuccess);
            chkHomeSuccess.Checked = Convert.ToBoolean(isSuccess);
        }

        private void btnSxStop_Click(object sender, EventArgs e)
        {
            int nAxis = cmbAxes.SelectedIndex;
            CEIP.cemSxStop(nAxis, 1, 0);
        }

        private void btnSxStopEmg_Click(object sender, EventArgs e)
        {
            int nAxis = cmbAxes.SelectedIndex;
            CEIP.cemSxStopEmg(nAxis);
        }

        // 본 함수는 완료시까지 HomeWaitDone 에서 잡고 있으며 따라서 동작중 강제로 중지를 원할시 윈도우 이벤트 특성 상 Stop 또는 Emg Stop 버튼이 바로 적용되지 않음(두번에 동작 됨)에 유의 하세요.
        // 따라서 모든 이송은 원래는 단발 쓰레드 함수를 통해 구현해 주시면 좋습니다. 즉 별도의 쓰레드 함수를 통해 독립적으로 이송작업 수행을 하면서 메인창에서는 다른 이벤트를 처리
        // 할수 있도록 해주는 것이 좋습니다.
        private void btnGoHome_Click(object sender, EventArgs e)
        {
            int nAxis = cmbAxes.SelectedIndex;
            CEIP.cemHomeSpeedPattern_Set(nAxis, (int)CEIP._TCemSpeedMode.cemSMODE_S, 1000, 5000, 5000, 200);
            CEIP.cemHomeConfig_Set(nAxis, 1, (int)CEIP._TCemDir.cemDIR_N, 0, 1000, 0); // 반드시 ESC Dist 는 100 정도 이상 충분히 줘야 이미 원점복귀가 된 상태에서 원점센서를 탈출 할 수 있습니다.
            CEIP.cemHomeMoveStart(nAxis);
            CEIP.cemHomeWaitDone(nAxis, 0); // 원점 복귀 완료 시까지 함수 반환이 안됩니다.(여기서 잡고 있습니다)

#if false // 참고 코드
            CEIP.cemHomeMove(nAxis, false);
#endif

#if false // 참고 코드
            CEIP.cemHomeMoveStart(nAxis);
            while (1)
            {
                int nIsBusy = 0;
                CEIP.cemHomeIsBusy(nAxis, ref nIsBusy);
                if (nIsBusy != 1) break;
            }
#endif
        }

        private void chkServoOnAxis0_CheckedChanged(object sender, EventArgs e)
        {
            int bOn = chkServoOnAxis0.Checked ? 1:0;
            CEIP.cemGnServoOn_Set((int)CEIP._TCemAxis.cemX1, bOn);
        }

        private void chkServoOnAxis1_CheckedChanged(object sender, EventArgs e)
        {
            int bOn = chkServoOnAxis1.Checked ? 1 : 0;
            CEIP.cemGnServoOn_Set((int)CEIP._TCemAxis.cemY1, bOn);
        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            // 디바이스 언로드
            CEIP.ceGnUnload();
            timer1.Enabled = false;
        }

        private void btnAlmReset_Click(object sender, EventArgs e)
        {
            int nAxis = cmbAxes.SelectedIndex;

            CEIP.cemGnAlarmReset(nAxis, 1);
            System.Threading.Thread.Sleep(500); // 이 시간은 서보AMP 의 알람리셋 입력 신호의 유효 Active Pulse Width 에 따라 달라집니다.
            CEIP.cemGnAlarmReset(nAxis, 0);
        }
    }
}
