﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using CEIP = ComiLib.cEIP.SafeNativeMethods;

namespace cEIP_SerialCom
{
    public partial class cEIP_SerialCom : Form
    {
        int g_nTotNodes = 0;
        int g_nTotSerialChannels = 0;
        bool g_bThreadStop = true;
        bool g_bIsProgAvailable = false;
        System.Threading.Thread g_ThFunc = null;

        const string REQUEST_STR	= "3A 30 41 30 33 32 33 38 44 30 30 30 31 34 32 0d 0a"; // AZBIL SDC 온도컨트롤러 명령1
        const string REQUEST_STR2   = "3A 30 41 30 33 30 33 45 39 30 30 30 32 30 35 0d 0a"; // AZBIL SDC 온도컨트롤러 명령2

        public cEIP_SerialCom()
        {
            CheckForIllegalCrossThreadCalls = false;
            InitializeComponent();
        }

        private int OpenComPort()
        {
            //string sVal;
            int tempInt = 0;
            int nNodeId = 0, nCh = 0, nSerType = 0, nBaudId = 0, nDataBits = 0, nStopBits = 0, nParity = 0;

            tempInt = cmbNodeID.SelectedIndex;
            //sVal.Trim();

            nNodeId = cmbNodeID.SelectedIndex;

            //sVal = (string)cmbSerCh.SelectedItem;
            //sVal.Trim();

            nCh = cmbSerCh.SelectedIndex;

            nSerType = cmbSerType.SelectedIndex + 3; // typedef enum _TCommProtocol {CE_COMM_RS232=3, CE_COMM_RS422, CE_COMM_RS485} TCommProtocol;

            nBaudId = cmbBaudRate.SelectedIndex;

            nDataBits = cmbDataBits.SelectedIndex + 7; // 실제 5 ~ 8의 값을 입력.

            nStopBits = cmbStopBits.SelectedIndex + 1; // 실제 1 ~ 2의 값을 입력.

            nParity = cmbParity.SelectedIndex;

            int nResult = CEIP.cesOpenPort(nNodeId + 1, nCh, nSerType, nBaudId, nDataBits, nStopBits, nParity);

            return nResult;
        }

        private bool CheckSC04AExist()
        {
            cmbSerCh.SelectedIndex = 2; // 0번(PE 모듈내 COM Port) 만 아니면 된다.

            return ((OpenComPort() == CEIP.ceERR_NONE) ? true : false);
        }

        private bool NodeScan()
        {
            const int REAL_NODES = 1;
            int IsSearchedDevice = 0;
            
            CEIP.ceGnIsSearchedDevice(ref IsSearchedDevice);
            if (Convert.ToBoolean(IsSearchedDevice)) CEIP.ceGnReSearchDevice(REAL_NODES, 1000, 0, ref g_nTotNodes);
            else CEIP.ceGnSearchDevice(REAL_NODES, 1000, 0, ref g_nTotNodes);

            if (g_nTotNodes < 1)
            {
                MessageBox.Show("검색된 노드가 하나도 없습니다! 네트워크 연결을 확인 하세요");
                return false;
            }
            else
            {
                return CheckSC04AExist();
            }
        }

        private void InitGui()
        {
            cmbNodeID.SelectedIndex = 0;

            cmbSerCh.SelectedIndex = 2;
            cmbSerType.SelectedIndex = 0;
            cmbBaudRate.SelectedIndex = 1;
            cmbDataBits.SelectedIndex = 1;
            cmbStopBits.SelectedIndex = 0;
            cmbParity.SelectedIndex = 0;

            txtSend.Text  = REQUEST_STR;
        }

        private void MainForm_Load(object sender, EventArgs e)
        {
            // 디바이스 로드
            if (CEIP.ceGnLoad() != CEIP.ceERR_NONE)
            {
                MessageBox.Show("cEIP Device 로드 실패!");
                g_bIsProgAvailable = false;
            }
            else
            {
                MessageBox.Show("cEIP Device 로드 성공!");
                InitGui();
                g_bIsProgAvailable = true;
                timer1.Enabled = true;
            }
        }

        private void btnNodeScan_Click(object sender, EventArgs e)
        {
            NodeScan();
            
            InitGui();
        }

        private void btnOpenPort_Click(object sender, EventArgs e)
        {
            string sVal;
            int nResult = OpenComPort();
            if (nResult != CEIP.ceERR_NONE)
            {
                sVal = string.Format("Failed to cesOpenPort. Error Code = {0}", nResult);
                MessageBox.Show(sVal);
            }
            else
            {
                MessageBox.Show("Success to cesOpenPort");
            }
        }

        private void ThSerial()
        {
	        if(!g_bIsProgAvailable) return;

	        int nIdx=0;

	        int nReadBytes = 0;
	        int i;
	
	        string sVal, sTmp;
	        int nNodeId=0, nCh=0;

	        sVal = (string)cmbNodeID.SelectedItem;
	        sVal.Trim();
	        nNodeId = Convert.ToInt32(sVal);
	
	        sVal = (string)cmbSerCh.SelectedItem;
	        sVal.Trim();
	        nCh = Convert.ToInt32(sVal);

	        sVal = txtSend.Text;
	        sVal.Trim();

	        char []szRequest = new char[20];

            // 전송할 패킷 문자열을 생성하기 위한 스트링 조작이므로 아래 부분은 알맞게 변경해서 사용하시면 됩니다.
	        for(i=0; i<sVal.Length; i+=3){
		        if(i< (sVal.Length - 1)){
			        sTmp = string.Format("0x{0}{1}", sVal[i], sVal[i+1]);
                    //szRequest[nIdx++] = Convert.ToChar(sTmp);
                    szRequest[nIdx++] = (char)Convert.ToInt32(sTmp, 16);
                }
	        }

	        int nWriteBytes = 0;
	        int nResult = CEIP.cesWriteString(nNodeId, nCh, szRequest.Length, szRequest, ref nWriteBytes);
	        int nSendSize = 0;

	        if(nResult != CEIP.ceERR_NONE){
		        sVal = string.Format("Failed to cesWriteString. Error Code = {0}", nResult);
		        MessageBox.Show(sVal);
	        }else{
                //MessageBox.Show("Success to cesWriteString");

		        nSendSize = 0;

		        nResult = CEIP.cesCommCommit(nNodeId, nCh, ref nSendSize);

		        if(nResult != CEIP.ceERR_NONE){
			        sVal = string.Format("Failed to cesCommCommit. Error Code = {0}", nResult);
                    MessageBox.Show(sVal);
			        return;
		        }
	        }	

	        return;
        }

        private void btnSend_Click(object sender, EventArgs e)
        {
            g_ThFunc = new System.Threading.Thread(new System.Threading.ThreadStart(ThSerial));
            g_ThFunc.Start();
        }

        private void btnReceive_Click(object sender, EventArgs e)
        {
	        string sVal;
	        int nNodeId=0, nCh=0;
	
	        sVal = (string)cmbNodeID.SelectedItem;
	        sVal.Trim();

	        nNodeId = Convert.ToInt32(sVal);
	
	        sVal = (string)cmbSerCh.SelectedItem;
	        sVal.Trim();

	        nCh = Convert.ToInt32(sVal);

	        uint nBufSize = 0;
	        CEIP.cesGetUnreadDataSize(nNodeId, nCh, ref nBufSize);
            System.Text.StringBuilder szReceiveBuf = new System.Text.StringBuilder();

            //for(int i=0; i<100; i++) szReceiveBuf[i] = '\0';
	
	        int nReadBytes = 0;
	        CEIP.cesReadString(nNodeId, nCh, Convert.ToInt32(nBufSize), szReceiveBuf, ref nReadBytes);
	
	        txtRecv.Text = string.Format("{0}", szReceiveBuf);
        }

        private void btnClosePort_Click(object sender, EventArgs e)
        {
            if(g_ThFunc != null)
            {
                g_ThFunc.Abort();
                g_bThreadStop = true;
            }
        }

        private void btnResetBuffer_Click(object sender, EventArgs e)
        {
            string sVal;
            int nNodeId=0, nCh=0;

            sVal = (string)cmbNodeID.SelectedItem;
            sVal.Trim();

            nNodeId = Convert.ToInt32(sVal);

            sVal = (string)cmbSerCh.SelectedItem;
            sVal.Trim();

            nCh = Convert.ToInt32(sVal);

            CEIP.cesTxReset(nNodeId, nCh);
            CEIP.cesRxReset(nNodeId, nCh);
        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            if (!g_bIsProgAvailable) return;

            string sVal;
            int nNodeId = 0, nCh = 0, nIsReady = 0;

            sVal = (string)cmbNodeID.SelectedItem;
            sVal.Trim();

            //nNodeId = Convert.ToInt32(sVal);

            sVal = (string)cmbSerCh.SelectedItem;
            sVal.Trim();

            //nCh = Convert.ToInt32(sVal);

            // 시리얼 통신 수신 데이터 존재 여부 체크
            CEIP.cesIsDataReady(nNodeId, nCh, ref nIsReady);
            chkDataReady.Checked = Convert.ToBoolean(nIsReady);
        }
    }
}
