﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using CEIP = ComiLib.cEIP.SafeNativeMethods;

namespace cEIP_IxLine
{
    public partial class cEIP_IxLine : Form
    {
        const int EXPECTED_NODECOUNT = 1; // 이 값은 실제 노드(ceNM-SE) 개수를 입력하면 됩니다.
        const int TIMEOUT = (1 * 100); // 노드 하나당 100ms
        int nSelAxis = 0;
        private const int IX_MAP_IDX = 0;  // Interpolation Map Index
        private bool m_bIsIxMapValid = false;
        int nNodes = 0;
        int nNumAxes = 0;
        int nReturn = 0;

        //bool g_bIsDeviceAvailable = false;

        public cEIP_IxLine()
        {
            InitializeComponent();
        }

        public int SelAxis
        {
            get { return nSelAxis; }
            set { nSelAxis = value; }
        }

        private void IxLine_Load(object sender, EventArgs e)
        {
            int nResult = CEIP.ceERR_NONE; // ceERR_NONE(0) 은 실패가 아니라 Error 가 없음을 의미, 에러 발생 시 마이너스(-) 값으로 에러 코드 반환

            // 디바이스 로드
            nResult = CEIP.ceGnLoad();
            if (nResult != CEIP.ceERR_NONE)
            {
                MessageBox.Show("Can Not Load Device");
            }
            else // 디바이스 로드 성공
            {
                RescanAndUpdate();
            }
        }

        private bool RescanAndUpdate()
        {
            // 타이머 동작을 먼저 멈춘다.(노드 재스캔 중 모션 함수 수행 금지)
            timer1.Enabled = false;

            int nResult = CEIP.ceERR_NONE; // ceERR_NONE(0) 은 실패가 아니라 Error 가 없음을 의미, 에러 발생 시 마이너스(-) 값으로 에러 코드 반환

            int nIsSearchedDev = 0;
            int nNumNodes = 0;

            //이미 검색된 노드인지 확인합니다.
            CEIP.ceGnIsSearchedDevice(ref nIsSearchedDev);

            if (nIsSearchedDev == 0) // 노드 검색이 기존에 완료 안됨
            {
                //노드를 검색합니다.
                if ((nResult = CEIP.ceGnSearchDevice(EXPECTED_NODECOUNT, TIMEOUT, 0, ref nNumNodes)) < 0)
                {
                    //ShowCeSdkError(nResult);
                    timer1.Enabled = false;
                    return false;
                }
            }
            else
            {
                //노드를 재검색합니다.
                if ((nResult = CEIP.ceGnReSearchDevice(EXPECTED_NODECOUNT, TIMEOUT, 0, ref nNumNodes)) < 0)
                {
                    //ShowCeSdkError(nResult);
                    timer1.Enabled = false;
                    return false;
                }
            }
                        

            CEIP.ceGnTotalNode(ref nNodes);

            CEIP.ceGnTotalMotionChannel(ref nNumAxes);

            // UI 업데이트
            string sAxis;
            cmbAxesSel.Items.Clear();
            for (int i = 0; i < nNumAxes; i++)
            {
                sAxis = "Axis" + i;
                cmbAxesSel.Items.Add(sAxis);

                this.cmbXAxis.Items.Insert(i, "Axis" + i.ToString());
                this.cmbYAxis.Items.Insert(i, "Axis" + i.ToString());
            }

            if (nNumAxes > 0)
            {
                //g_bIsDeviceAvailable = true;
                MessageBox.Show("노드 스캔 및 모션 모듈 인식 성공");
                cmbAxesSel.SelectedIndex = 0;
            }
            else
            {
                //g_bIsDeviceAvailable = false;
                MessageBox.Show("모션 모듈이 하나도 인식되지 않았습니다. 노드 연결 및 IP 설정을 점검하거나 스캔을 다시 시도하세요.");
                timer1.Enabled = false;

                return false;
            }
                        
            // 축선택의 기본값으로는 0번과 1번 두 축이 선택된 것으로 한다.
            this.cmbXAxis.SelectedIndex = 0;
            this.cmbYAxis.SelectedIndex = 1;

            // 초기 속도 모드
            this.cmbMstSpdMode.SelectedIndex = (int)CEIP._TCemSpeedMode.cemSMODE_S;
            this.cmbVecSpdMode.SelectedIndex = (int)CEIP._TCemSpeedMode.cemSMODE_S;

            // 초기 속도 패턴 설정
            for (int i = 0; i < nNumAxes; i++)
            {
                CEIP.cemSxOptIniSpeed_Set(i, 0);                
            }
            
            // 각종 초기화 및 로직 설정 => 기본적으로 서보알람과, EL(+) 및 EL(-) 센서가 OFF 상태이어야 CW 또는 CCW 방향으로 이송이 가능합니다. EL 센서인 경우는 해당 이송방향에 대한 +/-EL 상태에 따라 이송 안됨.
            // 0번축 설정
            CEIP.cemCfgMioProperty_Set(0, (int)CEIP.MioPropId.ALM_LOGIC, (int)CEIP.SigLogic.LOGIC_A); // 알람 로직은 보통 Logic_B 입니다.
            CEIP.cemCfgMioProperty_Set(0, (int)CEIP.MioPropId.EL_LOGIC, (int)CEIP.SigLogic.LOGIC_A); // 리미트센서 로직은 보통 Logic_B 입니다.
            CEIP.cemCfgMioProperty_Set(0, (int)CEIP.MioPropId.INP_EN, 0); // 서보로부터 INP(Inposition) 신호를 받지 않도록 설정. 만일 이 값이 Enable(1) 이면 Command Pulse 가 
            // 모두 출력이 되더라도 서보로부터 INP 를 받아야 이송완료로 처리되며 그렇지 않을 경우 이송완료를 무한 기다리는 상태가 되므로 주의.
            CEIP.cemCfgOutMode_Set(0, (int)CEIP._TCemOutMode.cemOMODE_CWCCW0); // 모터 드라이버의 지령펄스 입력 모드와 동일하게 맞춰야 펄스 및 방향이 올바르게 동작합니다.
            CEIP.cemCfgInMode_Set(0, (int)CEIP._TCemInMode.cemIMODE_AB4X, 0); // 모터 드라이버로부터의 피드백 펄스에 채배(1, 2 또는 4채배 가능)를 결정합니다.

            CEIP.cemCfgFilter_Set(0, 1); // ORG, EL 등 센서 신호에 대해 노이즈 필터링 할 수 있도록 처리
            CEIP.cemCfgFilterAB_Set(0, (int)CEIP._TCemABFilter.cemAB_ENC, 1); // 엔코더 입력 EA, EB 신호에 대해 노이즈 필터링 할 수 있도록 처리

            // 1번축 설정
            CEIP.cemCfgMioProperty_Set(1, (int)CEIP.MioPropId.ALM_LOGIC, (int)CEIP.SigLogic.LOGIC_A); // 알람 로직은 보통 Logic_B 입니다.
            CEIP.cemCfgMioProperty_Set(1, (int)CEIP.MioPropId.EL_LOGIC, (int)CEIP.SigLogic.LOGIC_A); // 리미트센서 로직은 보통 Logic_B 입니다.
            CEIP.cemCfgMioProperty_Set(1, (int)CEIP.MioPropId.INP_EN, 0); // 서보로부터 INP(Inposition) 신호를 받지 않도록 설정. 만일 이 값이 Enable(1) 이면 Command Pulse 가 
            // 모두 출력이 되더라도 서보로부터 INP 를 받아야 이송완료로 처리되며 그렇지 않을 경우 이송완료를 무한 기다리는 상태가 되므로 주의.
            CEIP.cemCfgOutMode_Set(1, (int)CEIP._TCemOutMode.cemOMODE_CWCCW0); // 모터 드라이버의 지령펄스 입력 모드와 동일하게 맞춰야 펄스 및 방향이 올바르게 동작합니다.
            CEIP.cemCfgInMode_Set(1, (int)CEIP._TCemInMode.cemIMODE_AB4X, 0); // 모터 드라이버로부터의 피드백 펄스에 채배(1, 2 또는 4채배 가능)를 결정합니다.

            CEIP.cemCfgFilter_Set(1, 1); // ORG, EL 등 센서 신호에 대해 노이즈 필터링 할 수 있도록 처리
            CEIP.cemCfgFilterAB_Set(1, (int)CEIP._TCemABFilter.cemAB_ENC, 1); // 엔코더 입력 EA, EB 신호에 대해 노이즈 필터링 할 수 있도록 처리
            
            timer1.Enabled = true;

            return true;
        }

        private void btnRescan_Click(object sender, EventArgs e)
        {
            RescanAndUpdate();
        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            int nServoState = 0;

            if (this.rdoVecSpdMode.Checked)
            {
                this.txtMstAxisCompare.Text = "Master 축 확인";
            }
            else
            {
                if (Convert.ToDouble(this.txtPos00.Text) >= Convert.ToDouble(this.txtPos01.Text))
                {
                    this.txtMstAxisCompare.Text = "Master 축은 X 축입니다.";
                }
                else
                {
                    this.txtMstAxisCompare.Text = "Master 축은 Y 축입니다.";
                }
            }

            CEIP.cemGnServoOn_Get(SelAxis, ref nServoState);
            if (nServoState == (int)CEIP.CE_FALSE)
            {
                btnServoOn.BackColor = Color.NavajoWhite;
                btnServoOn.Text = "Servo OFF";
            }
            else
            {
                btnServoOn.BackColor = Color.DarkOrange;
                btnServoOn.Text = "Servo ON";
            }
        }

        private void btnServoOn_Click(object sender, EventArgs e)
        {
            int nServoState=0;
            CEIP.cemGnServoOn_Get(SelAxis, ref nServoState);

            if (nServoState == 0)
            {
                CEIP.cemGnServoOn_Set(SelAxis, (int)CEIP.CE_TRUE);
            }
            else
            {
                CEIP.cemGnServoOn_Set(SelAxis, (int)CEIP.CE_FALSE);
            }
        }

        private void cmbAxesSel_SelectedIndexChanged(object sender, EventArgs e)
        {
            SelAxis = cmbAxesSel.SelectedIndex;
        }

        private void OnSpeedModeChanged(object sender, EventArgs e)
        {
            if (sender == rdoMstSpdMode)
            {
                this.cmbMstSpdMode.Enabled = true;
                this.txtMstVelSpd.Enabled = true;
                this.txtMstAccSpd.Enabled = true;
                this.txtMstDecSpd.Enabled = true;

                this.cmbVecSpdMode.Enabled = false;
                this.txtVecVelSpd.Enabled = false;
                this.txtVecAccSpd.Enabled = false;
                this.txtVecDecSpd.Enabled = false;
            }
            else if (sender == rdoVecSpdMode)
            {
                this.cmbMstSpdMode.Enabled = false;
                this.txtMstVelSpd.Enabled = false;
                this.txtMstAccSpd.Enabled = false;
                this.txtMstDecSpd.Enabled = false;

                this.cmbVecSpdMode.Enabled = true;
                this.txtVecVelSpd.Enabled = true;
                this.txtVecAccSpd.Enabled = true;
                this.txtVecDecSpd.Enabled = true;
            }
        }

        //private void btnStop_Click(object sender, EventArgs e)
        //{
        //    // 보간맵의 축설정이 아직 이루어지지 않은 경우에는 에러를 유발하므로 바로 리턴한다. //
        //    if (!m_bIsIxMapValid)
        //        return;

        //    if (CMDLL.cmmIxStop(IX_MAP_IDX, (int)CEIP._TCmBool.cmTRUE, (int)CEIP._TCmBool.cmFALSE) != CEIP.cmERR_NONE)
        //    {
        //        CMDLL.cmmErrShowLast(this.Handle);
        //    }

        //    btnGo.Enabled = true;
        //}

        private void btnGo_Click(object sender, EventArgs e)
        {
            btnGo.Enabled = false;

            int nAxisX = this.cmbXAxis.SelectedIndex;
            int nAxisY = this.cmbYAxis.SelectedIndex;

            // X축과 Y축이 같은 축번호이면 에러 치리
            if (nAxisX == nAxisY)
            {
                MessageBox.Show("Error: X and Y axes must be different from each other!", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                this.btnGo.Enabled = true;
                return;
            }

            int dwAxesMask1 = 0x0, dwAxesMask2 = 0x0;
            
            if (nAxisX < 32)
                dwAxesMask1 |= (1 << nAxisX); // X축 번호가 0~31인 경우에는 Mask1의 비트를 셋팅하고...
            else
                dwAxesMask2 |= (1 << (nAxisX - 32)); // X축 번호가 32~63인 경우에는 Mask2의 비트를 셋팅한다...

            if (nAxisY < 32)
                dwAxesMask1 |= (1 << nAxisY); // Y축 번호가 0~31인 경우에는 Mask1의 비트를 셋팅하고...
            else
                dwAxesMask2 |= (1 << (nAxisY - 32)); // Y축 번호가 32~63인 경우에는 Mask2의 비트를 셋팅한다...

            if (CEIP.cemIxMapAxes(IX_MAP_IDX, nNodes, dwAxesMask1, dwAxesMask2) == CEIP.ceERR_NONE)
                m_bIsIxMapValid = true;

            // Speed Setting //
            if (this.rdoVecSpdMode.Checked)
            {	// Vector Mode 이면..
                int nVectSpeedMode = this.cmbVecSpdMode.SelectedIndex;
                double fVectVel = Convert.ToDouble(this.txtVecVelSpd.Text);
                double fVectAcc = Convert.ToDouble(this.txtVecAccSpd.Text);
                double fVectDec = Convert.ToDouble(this.txtVecDecSpd.Text);

                CEIP.cemIxSpeedPattern_Set(IX_MAP_IDX, (int)CEIP.CE_TRUE, nVectSpeedMode, fVectVel, fVectAcc, fVectDec);
            }
            else
            {
                int nMastSpeedMode = this.cmbMstSpdMode.SelectedIndex;

                double fMastVel = Convert.ToDouble(this.txtMstDecSpd.Text);
                double fMastAcc = Convert.ToDouble(this.txtMstAccSpd.Text);
                double fMastDec = Convert.ToDouble(this.txtMstDecSpd.Text);

                CEIP.cemIxSpeedPattern_Set(IX_MAP_IDX, (int)CEIP.CE_TRUE, nMastSpeedMode, fMastVel, fMastAcc, fMastDec);
            }

            double fPosDataX = Convert.ToDouble(this.txtPos00.Text);
            double fPosDataY = Convert.ToDouble(this.txtPos01.Text);

            double[] afPosList = new double[2];
            afPosList[0] = fPosDataX;
            afPosList[1] = fPosDataY;

            if (this.rdoAbsPosMode.Checked)
            {
                nReturn = CEIP.cemIxLineTo(IX_MAP_IDX, afPosList, (int)CEIP.CE_FALSE);                
            }
            else
            {
                nReturn = CEIP.cemIxLine(IX_MAP_IDX, afPosList, (int)CEIP.CE_FALSE/* This must be TRUE in Thread */);
            }
        }

        private void btnStop_Click(object sender, EventArgs e)
        {
            
            // 보간맵의 축설정이 아직 이루어지지 않은 경우에는 에러를 유발하므로 바로 리턴한다. //
            if (!m_bIsIxMapValid)
                return;

            nReturn = CEIP.cemIxStop(IX_MAP_IDX);
        }
        
    }
}
